/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    Armar4::api
* @author     Nikolaus Vahrenkamp
* @date       2012 Nikolaus Vahrenkamp
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#include <Core/core/Component.h>
#include <Core/core/system/ImportExportComponent.h>
#include <Core/statechart/Statechart.h>
#include <Core/robotstate/remote/RemoteRobot.h>
#include <Core/robotstate/remote/ArmarPose.h>


//#include <VirtualRobot/VirtualRobot.h>

#include "RobotStatechartContext.h"

namespace armarx
{
    // ****************************************************************
    // Implementation of Component
    // ****************************************************************
    void RobotStatechartContext::onInitStatechartContext()
    {
//        StatechartContext::onInitStatechart();
        ARMARX_LOG << eINFO << "Init RobotStatechartContext" << flush;

        kinematicUnitObserverName = getProperty<std::string>("KinematicUnitObserverName").getValue();

        usingProxy(getProperty<std::string>("KinematicUnitName").getValue());
        usingProxy("RobotStateComponent");
        usingProxy(kinematicUnitObserverName);

        if( getProperty<std::string>("HandUnits").isSet())
        {
            std::string handUnitsProp = getProperty<std::string>("HandUnits").getValue();
            std::vector<std::string> handUnitList;
            boost::split(handUnitList, handUnitsProp, boost::is_any_of(","));
            for(size_t i = 0; i < handUnitList.size(); i++)
            {
                boost::algorithm::trim(handUnitList.at(i));
                usingProxy(handUnitList.at(i));
            }
        }
    }


    void RobotStatechartContext::onConnectStatechartContext()
    {
//        StatechartContext::onConnectStatechart();
        ARMARX_LOG << eINFO << "Starting RobotStatechartContext" << flush;

        // retrieve proxies
        robotStateComponent = getProxy<RobotStateComponentInterfacePrx>("RobotStateComponent");
        kinematicUnitPrx = getProxy<KinematicUnitInterfacePrx>(getProperty<std::string>("KinematicUnitName").getValue());
        kinematicUnitObserverPrx = getProxy<KinematicUnitObserverInterfacePrx>(kinematicUnitObserverName);
        ARMARX_LOG << eINFO << "Fetched proxies" <<  kinematicUnitPrx << " " << robotStateComponent << flush;

        if( getProperty<std::string>("HandUnits").isSet())
        {
            std::string handUnitsProp = getProperty<std::string>("HandUnits").getValue();
            std::vector<std::string> handUnitList;
            boost::split(handUnitList, handUnitsProp, boost::is_any_of(","));
            for(size_t i = 0; i < handUnitList.size(); i++)
            {
                boost::algorithm::trim(handUnitList.at(i));
                handUnits[handUnitList.at(i)] = getProxy<HandUnitInterfacePrx>(handUnitList.at(i));
            }
        }
		// initialize remote robot
        remoteRobot.reset(new RemoteRobot(robotStateComponent->getSynchronizedRobot()));
        ARMARX_LOG << eINFO << "Created remote robot" << flush;
    }

    PropertyDefinitionsPtr RobotStatechartContext::createPropertyDefinitions()
    {
        return PropertyDefinitionsPtr(new RobotStatechartContextProperties(
                                               getConfigIdentifier()));
    }

   /* const VirtualRobot::RobotPtr armarx::Armar4Context::getRobot()
    {
        return remoteRobot;
    }*/

}

