/**
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::
* @author     Mirko Waechter ( mirko.waechter at kit dot edu)
* @date       2013
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/
#ifndef _ARMARX_TCPCONTROLUNITOBSERVER_H
#define _ARMARX_TCPCONTROLUNITOBSERVER_H

#include <Core/observers/Observer.h>
#include <RobotAPI/interface/units/TCPControlUnit.h>

namespace armarx {

    class TCPControlUnitObserverPropertyDefinitions:
            public ComponentPropertyDefinitions
    {
    public:
        TCPControlUnitObserverPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineRequiredProperty<std::string>("TCPControlUnitName","Name of the TCPControlUnit");
        }
    };

    class TCPControlUnitObserver :
            virtual public Observer,
            virtual public TCPControlUnitObserverInterface
    {
    public:
        TCPControlUnitObserver();

        // framework hooks
        virtual std::string getDefaultName() const { return "TCPControlUnitObserver"; }
        void onInitObserver();
        void onConnectObserver();

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual PropertyDefinitionsPtr createPropertyDefinitions();

        // TCPControlUnitListener interface
    public:
        void reportTCPPose(const FramedPoseBaseMap &poseMap, const Ice::Current &c = Ice::Current());
        void reportTCPVelocities(const FramedVector3Map &tcpTranslationVelocities, const FramedVector3Map &tcpOrientationVelocities, const Ice::Current &c = Ice::Current());

        Mutex dataMutex;
    };

}

#endif
